package org.litesoft.orsup.selection;

// Copyright Status:
//
// All Software available from LiteSoft.org (including this file) is
// hereby released into the public domain.
//
// It is free!  As in, you may use it freely in both commercial and
// non-commercial applications, bundle it with your software
// distribution, include it on a CD-ROM, list the source code in a book,
// mirror the documentation at your own web site, or use it in any other
// way you see fit.
//
// NO Warranty!
//
// All software is provided "as is".
//
// There is ABSOLUTELY NO WARRANTY OF ANY KIND: not for the design, fitness
// (for a particular purpose), level of errors (or lack thereof), or
// applicability of this software.  The entire risk as to the quality
// and performance of this software is with you.  Should this software
// prove defective, you assume the cost of all necessary servicing, repair
// or correction.
//
// In no event unless required by applicable law or agreed to in writing
// will any party who created or may modify and/or redistribute this
// software, be liable to you for damages, including any general,
// special, incidental or consequential damages arising out of the use or
// inability to use this software (including but not limited to loss of
// data or data being rendered inaccurate or losses sustained by you or
// third parties or a failure of this software to operate with any
// other programs), even if such holder or other party has been advised
// of the possibility of such damages.
//
// NOTE: Should you discover a bug, have a recogmendation for a change, wish
// to submit modifications, or wish to add new classes/functionality,
// please email them to:
//
//        changes@litesoft.org
//

/**
 * An abstract representation of a SQL <i>WHERE</i> clause.<p>
 *
 * Using the <b>WhereClauseFactory</b> a complex SQL <i>WHERE</i> clause
 * can be generated.  The concrete extensions of <b>WhereClause</b> generated
 * by the <b>WhereClauseFactory</b> can form a <i>tree</i> that represents any
 * arbitrarily complex expression.<p>
 *
 * Two methods are provided to translate a <i>tree</i> of <b>WhereClause</b>s
 * into <b>String</b>s:<p>
 * <pre>
 *      toString() ;
 *      toSQL() ;
 * </pre>
 *
 * The <i>toString()</i> is intended for debuging purposes.<p>
 *
 * If you need to create/support a specific SQL <i>WHERE</i> clause that
 * is not provided, you would need to extend the <b>WhereClauseFactory</b>,
 * and extend this class.  However, you will probably not extend this class
 * directly, but extend one of the following abstract helper classes:<p>
 * <ul>
 *      <a href="AbstractWhereClauseAssociativeList.html">AbstractWhereClauseAssociativeList</a><br>
 *      <a href="AbstractWhereClauseWrapper.html">AbstractWhereClauseWrapper</a><br>
 *      <a href="AbstractWhereClauseColumnReference.html">AbstractWhereClauseColumnReference</a><br>
 *      <a href="AbstractWhereClauseColumnAndValue.html">AbstractWhereClauseColumnAndValue</a><br>
 *      <a href="AbstractWhereClauseColumnAndTwoValues.html">AbstractWhereClauseColumnAndTwoValues</a><br>
 *      <a href="AbstractWhereClauseColumnAndLikeValue.html">AbstractWhereClauseColumnAndLikeValue</a><br>
 *      and unlikely but possibly:<br>
 *      <ul>
 *          <a href="AbstractWhereClauseColumnIsIn.html">AbstractWhereClauseColumnIsIn</a><br>
 *      </ul>
 * </ul><p>
 *
 * By using the <i>Type</i>s (<tt>public static final int</tt>s),
 * direct interpretation / alternate translation can be accomplished.<p>
 *
 * See <a href="WhereClauseFactory.html">WhereClauseFactory</a><p>
 *
 * Exceptions: All problems caught when the parameter(s) are checked (as
 * indicated/implied in the @param tags) will generate an IllegalArgumentException,
 * and means the API user has a problem.  If a NullPointerException (or some
 * others, like: ClassCastException or ArrayIndexOutOfBoundsException) is thrown,
 * it means the API developer has a problem.  Any Exception that is explicitly
 * thrown in the API, but unrelated to a parameter, will be listed in the throws
 * clause (and hopefully in the tag @throws).  These may (but probably won't) be
 * checked Exceptions.
 *
 * @author  George Smith
 * @version 1.03 02/07/02 Exception Policy, Use of IllegalArgument class.
 * @version 1.02 11/05/01 Completed? JavaDocs
 * @version 1.0 10/07/01
 */

public abstract class WhereClause
{
    /** Type for            Equals.         Instantiated as <a href="AbstractWhereClauseColumnAndValue.html">AbstractWhereClauseColumnAndValue</a> */
    public static final int EQUALS = 0;

    /** Type for            Less Than.      Instantiated as <a href="AbstractWhereClauseColumnAndValue.html">AbstractWhereClauseColumnAndValue</a> */
    public static final int LESSTHAN = 1;

    /** Type for            Greater Than.   Instantiated as <a href="AbstractWhereClauseColumnAndValue.html">AbstractWhereClauseColumnAndValue</a> */
    public static final int GREATERTHAN = 2;

    /** Type for            Between.        Instantiated as <a href="AbstractWhereClauseColumnAndTwoValues.html">AbstractWhereClauseColumnAndTwoValues</a> */
    public static final int BETWEEN = 3;

    /** Type for            Or.             Instantiated as <a href="AbstractWhereClauseAssociativeList.html">AbstractWhereClauseAssociativeList</a> */
    public static final int OR = 4;

    /** Type for            And.            Instantiated as <a href="AbstractWhereClauseAssociativeList.html">AbstractWhereClauseAssociativeList</a> */
    public static final int AND = 5;

    /** Type for            Not.            Instantiated as <a href="AbstractWhereClauseWrapper.html">AbstractWhereClauseWrapper</a> */
    public static final int NOT = 6;

    /** Type for   String   Contains.       Instantiated as <a href="AbstractWhereClauseColumnAndLikeValue.html">AbstractWhereClauseColumnAndLikeValue</a> */
    public static final int CONTAINS = 7;

    /** Type for   String   Starts With.    Instantiated as <a href="AbstractWhereClauseColumnAndLikeValue.html">AbstractWhereClauseColumnAndLikeValue</a> */
    public static final int STARTS_WITH = 8;

    /** Type for   String   Ends With.      Instantiated as <a href="AbstractWhereClauseColumnAndLikeValue.html">AbstractWhereClauseColumnAndLikeValue</a> */
    public static final int ENDS_WITH = 9;

    /** Type for            Is Null.        Instantiated as <a href="AbstractWhereClauseColumnReference.html">AbstractWhereClauseColumnReference</a> */
    public static final int IS_NULL = 10;

    /** Type for            Is In.          Instantiated as <a href="AbstractWhereClauseColumnIsIn.html">AbstractWhereClauseColumnIsIn</a> */
    public static final int IS_IN = 11;

    private int zType;

    /**
     * Constructor that simply sets the <b>Type</b> of this WhereClause.<p>
     *
     * Please see the <tt>public static final int</tt>s for the currently
     * supported <b>Type</b>s.<p>
     *
     * @param   pType    the Type for this WhereClause.
     */
    protected WhereClause( int pType )
    {
        zType = pType;
    }

    /**
     * Accessor for the <b>Type</b> of this WhereClause.<p>
     *
     * Please see the <tt>public static final int</tt>s for the currently
     * supported <b>Type</b>s.<p>
     *
     * @return the <b>Type</b>.
     */
    public final int getType()
    {
        return zType;
    }

    /**
     * Generate a Debug friendly representation.<p>
     *
     * @return a String for Debuging.
     */
    public final String toString()
    {
        StringBuffer sb = new StringBuffer( "Where " );
        toStringHelper( sb );
        return sb.toString();
    }

    /**
     * Generate a String that may be used in a SQL statement, where the
     * <b>WHERE</b> <i>clause</i> would go.<p>
     *
     * Note: This String can get quite large.<p>
     *
     * @return the <b>WHERE</b> <i>clause</i> for SQL statement as a String.
     */
    public final String toSQL()
    {
        StringBuffer sb = new StringBuffer( "WHERE " );
        toSqlHelper( sb );
        return sb.toString();
    }

    /**
     * Helper method for <b>toString()</b> that provides a more efficient
     * mechanism for the recursive decent of a WhereClause <i>tree</i>.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.<p>
     *
     * @see #toString()
     */
    protected abstract void toStringHelper( StringBuffer pSB );

    /**
     * Helper method for <b>toSQL()</b> that provides a more efficient
     * mechanism for the recursive decent of a WhereClause <i>tree</i>.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.<p>
     *
     * @see #toSQL()
     */
    protected abstract void toSqlHelper( StringBuffer pSB );

    /**
     * Helper method for <b>toSqlHelper()</b> that adds a prefix and then
     * parenthesizes a nested WhereClause.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.
     * @param   pPreFix  the prefix to append.
     * @param   pWhereClause  the nested WhereClause to wrap with parenthesizes
     *                          and append.<p>
     *
     * @see #toSqlHelper(StringBuffer)
     */
    protected static void toSqlHelperParenthesizer( StringBuffer pSB , String pPreFix , WhereClause pWhereClause )
    {
        pSB.append( pPreFix );
        pSB.append( ' ' );
        toSqlHelperParenthesizer( pSB , pWhereClause );
    }

    /**
     * Helper method for <b>toSqlHelper()</b> that adds a parenthesized
     * nested WhereClause.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.
     * @param   pWhereClause  the nested WhereClause to wrap with parenthesizes
     *                          and append.<p>
     *
     * @see #toSqlHelper(StringBuffer)
     */
    protected static void toSqlHelperParenthesizer( StringBuffer pSB , WhereClause pWhereClause )
    {
        pSB.append( '(' );
        pWhereClause.toSqlHelper( pSB );
        pSB.append( ')' );
    }

    /**
     * Helper method for <b>toStringHelper()</b> that adds a parenthesized
     * nested WhereClause.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.
     * @param   pWhereClause  the nested WhereClause to wrap with parenthesizes
     *                          and append.<p>
     *
     * @see #toStringHelper(StringBuffer)
     */
    protected static void toStringHelperParenthesizer( StringBuffer pSB , WhereClause pWhereClause )
    {
        pSB.append( '(' );
        pWhereClause.toStringHelper( pSB );
        pSB.append( ')' );
    }

    /**
     * Helper method for <b>toStringHelper()</b> that adds a prefix and then
     * parenthesizes a nested WhereClause.<p>
     *
     * @param   pSB   the StringBuffer to build the WhereClause into.
     * @param   pPreFix  the prefix to append.
     * @param   pWhereClause  the nested WhereClause to wrap with parenthesizes
     *                          and append.<p>
     *
     * @see #toStringHelper(StringBuffer)
     */
    protected static void toStringHelperParenthesizer( StringBuffer pSB , String pPreFix , WhereClause pWhereClause )
    {
        pSB.append( pPreFix );
        pSB.append( ' ' );
        toStringHelperParenthesizer( pSB , pWhereClause );
    }
}
